package checkers.metaquals;

import java.lang.annotation.*;

import javax.lang.model.type.TypeKind;

import checkers.types.AnnotatedTypeMirror;

import com.sun.source.tree.Tree;

/**
 * A meta-annotation that specifies the trees and types for which the framework
 * should automatically add that qualifier. These types and trees can be
 * specified via any combination of four fields.
 * 
 * For example, the {@code @checkers.quals.Nullable} annotation is annotated
 * with {@code @ImplicitFor(trees={Tree.Kind.NULL LITERAL})} to denote that
 * the framework should automatically apply {@code @Nullable} to all instances
 * of "null."
 * 
 * (Note: {@code @ImplicitFor} handling currently requires explicitly invoking
 * {@code checkers.types.TypeAnnotator} and {@code checkers.types.TreeAnnotator}
 * from within the checkers.types.AnnotatedTypeFactory.annotateImplicit method.
 * Automatic handling of {@code @ImplicitFor} will be added to a future release.)
 *
 */
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.ANNOTATION_TYPE)
public @interface ImplicitFor {
   
    /**
     * @return {@link Tree.Kind}s of trees for which an annotation should be
     *         implicitly added
     */
    Tree.Kind[] trees() default {};
    
    /**
     * @return {@link Class}es of trees for which an annotation should be
     *         implicitly added
     */
    Class<? extends Tree>[] treeClasses() default {};
    
    /**
     * @return {@link TypeKind}s of types for which an annotation should be
     *         implicitly added
     */
    TypeKind[] types() default {};
    
    /**
     * @return {@link Class}es of types for which an annotation should be
     *         implicitly added
     */
    Class<? extends AnnotatedTypeMirror>[] typeClasses() default {};
}
